////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// TCalendarBase \[Xt@C Ver.2.0                                       //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
unit CalendarBase;

interface

//Of[V̎gpEsgp؂ւRpCXCb`
{$DEFINE USE_GRADIENT}

uses
  Windows, Messages, DateUtils, Classes, Controls, ComCtrls, Graphics, Forms,
  TexturePanel, SpinEditEx, CsvTables;

const
  DEF_COLOR_WEEKDAY = clBlack;
  DEF_COLOR_HALFDAY = clBlue;
  DEF_COLOR_HOLIDAY = clRed;
  DEF_COLOR_SPECIAL = clGreen;
  DEF_COLOR_TITLE_BACK = clNavy;
  DEF_COLOR_TITLE_TEXT = clWhite;

type
  TCalendarBase = class;

  //tʁiExExEʓj
  TTypeOfDay = (todWeekday, todHalfday, todHoliday, todSpecial);
  TTypeOfDays = set of TTypeOfDay;

  //JԂ^CviPENEETEj
  TRecurFreq = (rfONCE, rfYEARLY, rfMONTHLY, rfWEEKLY, rfDAILY);

  //XPW[ACe
  TScheduleItem = class
  private
    { Private 錾 }
    SUMMARY: String;
    DESCRIPTION: String;
    CATEGORIES: TTypeOfDay;
    DTSTART: LongInt;
    DTEND: LongInt;
    RULE_FREQ: TRecurFreq;
    RULE_INTERVAL: Integer;
    RULE_BYDAY: TList;
    RULE_BYDAYPOS: TList;
    RULE_BYMONTHDAY: TList;
    RULE_BYYEARDAY: TList;
    RULE_BYWEEKNO: TList;
    RULE_BYMONTH: TList;
    RULE_BYSETPOS: TList;
    RULE_UNTIL: LongInt;
  private
    { Private 錾 }
    function GetTypeOfDay: TTypeOfDay;
    function GetItemText(AddDescript: Boolean): String;
  public
    { Public 錾 }
    constructor Create;
    destructor Destroy; override;
    function Matching(JD: LongInt; JDY, JDM, JDD: Word): Boolean; overload;
    function Matching(JDY, JDM, JDD: Word): Boolean; overload;
    function Matching(JD: LongInt): Boolean; overload;
    property TypeOfDay: TTypeOfDay read GetTypeOfDay;
    property ItemText[Index: Boolean]: String read GetItemText;
  end;

  //XPW[Xg
  TSchedules = class(TComponent)
  private
    { Private 錾 }
    FUpdateLocked: Boolean;
    FTypeOfDays: array[DayMonday..DaySunday] of TTypeOfDay;
    FMinDate: Longint;
    FMaxDate: Longint;
    FClientList: TList;
  protected
    { Protected 錾 }
    FMatchingList: TList;
    procedure Loaded; override;
    procedure OpenData(OpenDataSet: TCsvDataSet); virtual; abstract;
    procedure PostData(PostDataSet: TCsvDataSet); virtual; abstract;
    function MatchItem(JD: LongInt): TTypeOfDays;
    function GetTypeOfWeek(Index: Integer): TTypeOfDay;
    function GetMinDate: TDateTime;
    function GetMaxDate: TDateTime;
    procedure SetTypeOfWeek(Index: Integer; const Value: TTypeOfDay);
    procedure SetMinDate(const Value: TDateTime);
    procedure SetMaxDate(const Value: TDateTime);
  public
    { Public 錾 }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Update;
    procedure Repaint;
    procedure Synchronize(const Value: LongInt);
    procedure RegistClient(AClient: TCalendarBase);
    procedure UnRegistClient(AClient: TCalendarBase);
    function GetTypeOfDate(JD: LongInt): TTypeOfDays;
    function GetTextOfDate(JD: LongInt; WithDetail: Boolean): String;
    function CheckDateMinMax(const Value: LongInt): LongInt;
  published
    { Published 錾 }
    property TypeOfMon: TTypeOfDay index DayMonday    read GetTypeOfWeek write SetTypeOfWeek default todWeekday;
    property TypeOfTue: TTypeOfDay index DayTuesday   read GetTypeOfWeek write SetTypeOfWeek default todWeekday;
    property TypeOfWed: TTypeOfDay index DayWednesday read GetTypeOfWeek write SetTypeOfWeek default todWeekday;
    property TypeOfThu: TTypeOfDay index DayThursday  read GetTypeOfWeek write SetTypeOfWeek default todWeekday;
    property TypeOfFri: TTypeOfDay index DayFriday    read GetTypeOfWeek write SetTypeOfWeek default todWeekday;
    property TypeOfSat: TTypeOfDay index DaySaturday  read GetTypeOfWeek write SetTypeOfWeek default todHalfday;
    property TypeOfSun: TTypeOfDay index DaySunday    read GetTypeOfWeek write SetTypeOfWeek default todHoliday;
    property MinDate: TDateTime read GetMinDate write SetMinDate;
    property MaxDate: TDateTime read GetMaxDate write SetMaxDate;
  end;

  //J_[ptHg
  TCalendarFonts = class(TPersistent)
  private
    { Private 錾 }
    FWeekday: TColor;
    FHalfday: TColor;
    FHoliday: TColor;
    FSpecial: TColor;
    FTitleBack: TColor;
    FTitleText: TFont;
    function GetDaysBack: TColor;
    function GetDaysText: TFont;
    function GetColors(Index: TTypeOfDay): TColor;
    procedure SetDaysBack(const Value: TColor);
    procedure SetDaysText(const Value: TFont);
    procedure SetColors(Index: TTypeOfDay; const Value: TColor);
    procedure SetTitleBack(const Value: TColor);
    procedure TitleFontChanged(Sender: TObject);
  protected
    { Protected 錾 }
    FOwner: TCalendarBase;
  public
    { Public 錾 }
    constructor Create(AOwner: TCalendarBase);
    destructor Destroy; override;
  published
    { Published 錾 }
    property DaysBack: TColor read GetDaysBack write SetDaysBack;
    property DaysText: TFont read GetDaysText write SetDaysText;
    property Weekday: TColor index todWeekday read GetColors write SetColors default DEF_COLOR_WEEKDAY;
    property Halfday: TColor index todHalfday read GetColors write SetColors default DEF_COLOR_HALFDAY;
    property Holiday: TColor index todHoliday read GetColors write SetColors default DEF_COLOR_HOLIDAY;
    property Special: TColor index todSpecial read GetColors write SetColors default DEF_COLOR_SPECIAL;
    property TitleBack: TColor read FTitleBack write SetTitleBack default DEF_COLOR_TITLE_BACK;
    property TitleText: TFont read FTitleText write FTitleText;
  end;

  //J_[x[X
{$IFDEF USE_GRADIENT}
  TCalendarBase = class(TTextureControl)
{$ELSE}
  TCalendarBase = class(TBitmapControl)
{$ENDIF}
  private
    { Private 錾 }
    FNextBtn: TRapidBtnLite;
    FPrevBtn: TRapidBtnLite;
    FSchedules: TSchedules;
    FCalFonts: TCalendarFonts;
    FShowTitle: Boolean;
    FSyncDate: Boolean;
    FJulianDate: Longint;
    FFirstDayOfWeek: TCalDayOfWeek;
    FOnInternalExit: TNotifyEvent;
    FOnInternalChange: TNotifyEvent;
    FOnInternalSelect: TNotifyEvent;
    FOnDateChange: TNotifyEvent;
    FOnDateSelect: TNotifyEvent;
    FOnNextBtnClick: TNotifyEvent;
    FOnPrevBtnClick: TNotifyEvent;
    function GetCurrentDate: TDateTime;
    function GetCurrentYear: Word;
    function GetCurrentMonth: Word;
    function GetCurrentDay: Word;
    function GetShowButton: Boolean;
    procedure SetSchedules(Value: TSchedules);
    procedure SetCalendarFonts(const Value: TCalendarFonts);
    procedure SetCurrentDate(const Value: TDateTime);
    procedure SetCurrentYear(const Value: Word);
    procedure SetCurrentMonth(const Value: Word);
    procedure SetCurrentDay(const Value: Word);
    procedure SetJulianDate(const Value: Longint);
    procedure SetFirstDayOfWeek(const Value: TCalDayOfWeek);
    procedure SetShowButton(const Value: Boolean);
    procedure SetShowTitle(const Value: Boolean);
    procedure SetSynchroDate(const Value: Boolean);
    procedure ParentEnter(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure PrevBtnClick(Sender: TObject);
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMContextMenu(var Message: TWMContextMenu); message WM_CONTEXTMENU;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
    { Protected 錾 }
    procedure CreateParams(var Params: TCreateParams); override;
    function GetTitleHeight: Integer;
    function GetMousePointDate(X, Y: Integer; out NewValue: LongInt): Boolean; dynamic;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DoExit; override;
    procedure DoDateChange;
    procedure DoDateSelect;
    procedure DoNextBtnClick; virtual;
    procedure DoPrevBtnClick; virtual;
    procedure TitleResize;
    procedure DrawTitle(ATitle: String; ACanvas: TCanvas; ARect: TRect); dynamic;
    property OnInternalExit: TNotifyEvent read FOnInternalExit write FOnInternalExit;
    property OnInternalChange: TNotifyEvent read FOnInternalChange write FOnInternalChange;
    property OnInternalSelect: TNotifyEvent read FOnInternalSelect write FOnInternalSelect;
  public
    { Public 錾 }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ConvFirstDayOfWeek: Byte;
    function GetFontOfTOD(TOD: TTypeOfDays): TColor;
    function GetFontOfWeek(DOW: Byte): TColor;
    function GetFontOfDate(JD: LongInt): TColor;
    function GetTextOfDate(JD: LongInt; MultiLine: Boolean): String;
    procedure JumpTo(Y, M, D: Word);
    procedure JumpToToday;
    procedure JumpToNextDay;
    procedure JumpToPrevDay;
    procedure JumpToNextWeek;
    procedure JumpToPrevWeek;
    procedure JumpToNextMonth;
    procedure JumpToPrevMonth;
    procedure JumpToNextYear;
    procedure JumpToPrevYear;
    procedure JumpToIncMonth(Months: Word);
    procedure JumpToDecMonth(Months: Word);
    procedure JumpToStartOfWeek;
    procedure JumpToEndOfWeek;
    procedure JumpToStartOfMonth;
    procedure JumpToEndOfMonth;
    procedure JumpToStartOfHalf;
    procedure JumpToEndOfHalf;
    procedure JumpToStartOfYear;
    procedure JumpToEndOfYear;
    property CalendarFonts: TCalendarFonts read FCalFonts write SetCalendarFonts;
    property CurrentDate: TDateTime read GetCurrentDate write SetCurrentDate stored False;
    property CurrentYear: Word read GetCurrentYear write SetCurrentYear stored False;
    property CurrentMonth: Word read GetCurrentMonth write SetCurrentMonth stored False;
    property CurrentDay: Word read GetCurrentDay write SetCurrentDay stored False;
    property FirstDayOfWeek: TCalDayOfWeek read FFirstDayOfWeek write SetFirstDayOfWeek default dowLocaleDefault;
    property JulianDate: Longint read FJulianDate write SetJulianDate;
    property Schedules: TSchedules read FSchedules write SetSchedules;
    property ShowButton: Boolean read GetShowButton write SetShowButton default False;
    property ShowTitle: Boolean read FShowTitle write SetShowTitle default True;
    property SynchroDate: Boolean read FSyncDate write SetSynchroDate default False;
    property OnDateChange: TNotifyEvent read FOnDateChange write FOnDateChange;
    property OnDateSelect: TNotifyEvent read FOnDateSelect write FOnDateSelect;
    property OnNextBtnClick: TNotifyEvent read FOnNextBtnClick write FOnNextBtnClick;
    property OnPrevBtnClick: TNotifyEvent read FOnPrevBtnClick write FOnPrevBtnClick;
    property Color stored False;
    property Font stored False;
    property TabStop default True;
  end;


  //J_[phbLOtH[
  TCalendarDockForm = class(TCustomForm)
  private
    procedure CMDockNotification(var Message: TCMDockNotification); message CM_DOCKNOTIFICATION;
    procedure CMUnDockClient(var Message: TCMUnDockClient); message CM_UNDOCKCLIENT;
  protected
    procedure DoAddDockClient(Client: TControl; const ARect: TRect); override;
    procedure DoRemoveDockClient(Client: TControl); override;
  public
    constructor Create(AOwner: TComponent); override;
    property AutoScroll default False;
    property BorderStyle default bsNone;
    property FormStyle default fsStayOnTop;
  published
    property OnDeactivate;
  end;

implementation

uses SysUtils, JulianUtils;


////////////////////////////////////////////////////////////////////////////////
// TScheduleItem                                                              //
////////////////////////////////////////////////////////////////////////////////

//Create
constructor TScheduleItem.Create;
begin
  inherited Create;

  //
  SUMMARY := '';
  DESCRIPTION := '';
  CATEGORIES := todSpecial;
  DTSTART := MinJulianDay;
  DTEND := MinJulianDay;
  RULE_FREQ := rfONCE;
  RULE_INTERVAL := 0;
  RULE_BYDAY := TList.Create;
  RULE_BYDAYPOS := TList.Create;
  RULE_BYMONTHDAY := TList.Create;
  RULE_BYYEARDAY := TList.Create;
  RULE_BYWEEKNO := TList.Create;
  RULE_BYMONTH := TList.Create;
  RULE_BYSETPOS := TList.Create;
  RULE_UNTIL := MaxJulianDay;
end;

//Destroy
destructor TScheduleItem.Destroy;
begin
  RULE_BYDAY.Clear;
  RULE_BYDAYPOS.Clear;
  RULE_BYMONTHDAY.Clear;
  RULE_BYYEARDAY.Clear;
  RULE_BYWEEKNO.Clear;
  RULE_BYMONTH.Clear;
  RULE_BYSETPOS.Clear;
  FreeAndNil(RULE_BYDAY);
  FreeAndNil(RULE_BYDAYPOS);
  FreeAndNil(RULE_BYMONTHDAY);
  FreeAndNil(RULE_BYYEARDAY);
  FreeAndNil(RULE_BYWEEKNO);
  FreeAndNil(RULE_BYMONTH);
  FreeAndNil(RULE_BYSETPOS);

  inherited;
end;

//}b`O
function TScheduleItem.Matching(JD: LongInt; JDY, JDM, JDD: Word): Boolean;
var
  nYS, nMS, nDS, nYE, nME, nDE: Word;
  nIdx: Integer;
begin
  Result := False;

  //LԂɂi荞
  if JD < DTSTART then Exit;
  if JD >= RULE_UNTIL then Exit;
  if (RULE_FREQ = rfONCE)or(RULE_INTERVAL = 0) then
  begin
    if JD >= DTEND then Exit;
  end;

  DecodeJulianDay(DTSTART, nYS, nMS, nDS);
  DecodeJulianDay(DTEND, nYE, nME, nDE);

  //Nɂi荞
  if (RULE_FREQ = rfYEARLY)and(RULE_INTERVAL > 0) then
  begin
    if ((JDY - nYS) mod RULE_INTERVAL) > (nYE - nYS) then Exit;
    if nMS > nME then
    begin
      if (JDM < nMS)and(JDM > nMS) then Exit;
    end
    else
    begin
      if (JDM < nMS)or(JDM > nMS) then Exit;
    end;
    if nDS > nDE then
    begin
      if (JDD < nDS)and(JDD >= nDE) then Exit;
    end
    else
    begin
      if (JDD < nDS)or(JDD >= nDE) then Exit;
    end;
  end;

  //ɂi荞
  if (RULE_FREQ = rfMONTHLY)and(RULE_INTERVAL > 0) then
  begin
    if ((((JDY - nYS) * 12 + (JDM - nMS)) mod RULE_INTERVAL) > ((nYE - nYS) * 12 + (nME - nMS))) then Exit;
  end;
  if RULE_BYMONTH.Count > 0 then
  begin
    if RULE_BYMONTH.IndexOf(PInteger(JDM)) = -1 then
    begin
      //NłȂΏO
      if JDM <> 12 then Exit;
      //NȂΔN(-1)̎w`FbN
      if RULE_BYMONTH.IndexOf(PInteger(-1)) = -1 then Exit;
    end;
  end;

  //Tɂi荞
  if (RULE_FREQ = rfWEEKLY)and(RULE_INTERVAL > 0) then
  begin
    if ((JD - DTSTART) mod (RULE_INTERVAL * 7)) < (DTEND - DTSTART) then Exit;
  end;
  if RULE_BYSETPOS.Count > 0 then
  begin
    nIdx := (JDD + 6) div 7;
    if RULE_BYSETPOS.IndexOf(PInteger(nIdx)) = -1 then
    begin
      //ŏITłȂΏO
      if nIdx <> (DaysInAMonth(JDY, JDM) + 6) div 7 then Exit;
      //ŏITȂΔNŏIT(-1)̎w`FbN
      if RULE_BYSETPOS.IndexOf(PInteger(-1)) = -1 then Exit;
    end;
  end;
  if RULE_BYWEEKNO.Count > 0 then
  begin
    nIdx := WeekOfJulian(JD);
    if RULE_BYWEEKNO.IndexOf(PInteger(nIdx)) = -1 then
    begin
      //NŏITłȂΏO
      if nIdx <> WeekOfJulian(EncodeJulianDay(JDY, 12, 31)) then Exit;
      //NŏITȂΔNŏIT(-1)̎w`FbN
      if RULE_BYWEEKNO.IndexOf(PInteger(-1)) = -1 then Exit;
    end;
  end;

  //ɂi荞
  if (RULE_FREQ = rfDAILY)and(RULE_INTERVAL > 0) then
  begin
    if ((JD - DTSTART) mod RULE_INTERVAL) < (DTEND - DTSTART) then Exit;
  end;
  if RULE_BYMONTHDAY.Count > 0 then
  begin
    if RULE_BYMONTHDAY.IndexOf(PInteger(JDD)) = -1 then
    begin
      //łȂΏO
      if JDD <> DaysInAMonth(JDY, JDM) then Exit;
      //ȂΌ(-1)̎w`FbN
      if RULE_BYMONTHDAY.IndexOf(PInteger(-1)) = -1 then Exit;
    end;
  end;
  if RULE_BYYEARDAY.Count > 0 then
  begin
    if RULE_BYYEARDAY.IndexOf(PInteger(JD - StartOfTheYearJ(JD) + 1)) = -1 then
    begin
      //NłȂΏO
      if (JDM <> 12) or (JDD <> 31) then Exit;
      //NȂΔN(-1)̎w`FbN
      if RULE_BYYEARDAY.IndexOf(PInteger(-1)) = -1 then Exit;
    end;
  end;

  //jɂi荞
  if RULE_BYDAY.Count > 0 then
  begin
    nIdx := RULE_BYDAY.IndexOf(PInteger(DayOfTheWeekJ(JD)));
    if nIdx = -1 then Exit;
    //HT̗jɂi荞
    nIdx := Integer(RULE_BYDAYPOS[nIdx]);
    if nIdx > 0 then
    begin
      //HT
      if nIdx <> (JDD + 6) div 7 then Exit;
    end
    else if nIdx = -1 then
    begin
      //ŏIT
      if DaysInAMonth(JDY, JDM) - JDD < 7 then Exit;
    end;
  end;

  Result := True;
end;

function TScheduleItem.Matching(JD: LongInt): Boolean;
var
  nYJ, nMJ, nDJ: Word;
begin
  DecodeJulianDay(JD, nYJ, nMJ, nDJ);
  Result := Matching(JD, nYJ, nMJ, nDJ);
end;

function TScheduleItem.Matching(JDY, JDM, JDD: Word): Boolean;
begin
  Result := Matching(EncodeJulianDay(JDY, JDM, JDD), JDY, JDM, JDD);
end;

//tʎ擾
function TScheduleItem.GetTypeOfDay: TTypeOfDay;
begin
  case CATEGORIES of
  todWeekday:
    Result := todWeekday;
  todHoliday:
    Result := todHoliday;
  todHalfday:
    Result := todHalfday;
  else
    Result := todSpecial;
  end;
end;

//teLXg擾
function TScheduleItem.GetItemText(AddDescript: Boolean): String;
begin
  Result := SUMMARY;
  if (not AddDescript)or(DESCRIPTION = '') then Exit;
  Result := Result + #13#10 + DESCRIPTION;
end;


////////////////////////////////////////////////////////////////////////////////
// TSchedules                                                                 //
////////////////////////////////////////////////////////////////////////////////

//Create
constructor TSchedules.Create(AOwner: TComponent);
begin
  inherited;
  FUpdateLocked := False;
  FTypeOfDays[DayMonday] := todWeekday;
  FTypeOfDays[DayTuesday] := todWeekday;
  FTypeOfDays[DayWednesday] := todWeekday;
  FTypeOfDays[DayThursday] := todWeekday;
  FTypeOfDays[DayFriday] := todWeekday;
  FTypeOfDays[DaySaturday] := todHalfday;
  FTypeOfDays[DaySunday] := todHoliday;
  FMinDate := MinJulianDay;
  FMaxDate := MaxJulianDay;
  FClientList := TList.Create;
  FMatchingList := TList.Create;
end;

//Destroy
destructor TSchedules.Destroy;
var
  Cnt: Integer;
begin
  FUpdateLocked := True;

  for Cnt := 0 to FClientList.Count - 1 do
  begin
    if Assigned(FClientList.Items[Cnt]) then
    begin
      TCalendarBase(FClientList.Items[Cnt]).Schedules := nil;
    end;
  end;
  FClientList.Clear;
  for Cnt := 0 to FMatchingList.Count - 1 do
  begin
    TScheduleItem(FMatchingList.Items[Cnt]).Free;
  end;
  FMatchingList.Clear;

  FreeAndNil(FMatchingList);
  FreeAndNil(FClientList);
  inherited;
end;

//ǂݍ݌XPW[XV
procedure TSchedules.Loaded;
begin
  inherited;
  Update;
end;

//ŏ擾
function TSchedules.GetMinDate: TDateTime;
begin
  Result := JulianDayToDate(FMinDate);
end;

//ő擾
function TSchedules.GetMaxDate: TDateTime;
begin
  Result := JulianDayToDate(FMaxDate);
end;

//ŏݒ
procedure TSchedules.SetMinDate(const Value: TDateTime);
var
  TempDate: LongInt;
  Cnt: Integer;
begin
  TempDate := DateToJulianDay(Value);
  if FMinDate = TempDate then Exit;
  FMinDate := TempDate;
  if FMaxDate < TempDate then FMaxDate := TempDate;
  FUpdateLocked := True;
  try
    for Cnt := 0 to FClientList.Count - 1 do
    begin
      if Assigned(FClientList.Items[Cnt]) then
      begin
        if TCalendarBase(FClientList.Items[Cnt]).JulianDate < TempDate then
          TCalendarBase(FClientList.Items[Cnt]).JulianDate := TempDate;
      end;
    end;
  finally
    FUpdateLocked := False;
  end;
end;

//őݒ
procedure TSchedules.SetMaxDate(const Value: TDateTime);
var
  TempDate: LongInt;
  Cnt: Integer;
begin
  TempDate := DateToJulianDay(Value);
  if FMaxDate = TempDate then Exit;
  FMaxDate := TempDate;
  if FMinDate > TempDate then FMinDate := TempDate;
  FUpdateLocked := True;
  try
    for Cnt := 0 to FClientList.Count - 1 do
    begin
      if Assigned(FClientList.Items[Cnt]) then
      begin
        if TCalendarBase(FClientList.Items[Cnt]).JulianDate > TempDate then
          TCalendarBase(FClientList.Items[Cnt]).JulianDate := TempDate;
      end;
    end;
  finally
    FUpdateLocked := False;
  end;
end;

//XPW[XV
procedure TSchedules.Update;
var
  nCnt, nIdx: Integer;
  sVal: String;
  tCI: TScheduleItem;
  tDS: TCsvDataSet;
  tSL: TStringList;

  function DTStampToJulianDef(const AValue: String; const ADefault: LongInt): LongInt;
  var
    JD: LongInt;
    nYear, nMon, nDay: Integer;
  begin
    Result := ADefault;
    if AValue = '' then Exit;

    //N𕪉
    if not TryStrToInt(Copy(AValue, 1, 4), nYear) then Exit;
    if not TryStrToInt(Copy(AValue, 5, 2), nMon) then Exit;
    if not TryStrToInt(Copy(AValue, 7, 2), nDay) then Exit;

    //tɕϊ
    if not TryEncodeJulianDay(nYear, nMon, nDay, JD) then Exit;
    Result := JD;
  end;

  procedure SetIntArrayList(const AValue: String; var OLsit: TList);
  var
    nRCnt, nRVal: Integer;
    tRSL: TStringList;
  begin
    OLsit.Clear;
    if AValue = '' then Exit;
    tRSL := TStringList.Create;
    try
      tRSL.CommaText := AValue;
      for nRCnt := 0 to tRSL.Count - 1 do
      begin
        if TryStrToInt(tRSL.Strings[nRCnt], nRVal) then OLsit.Add(PInteger(nRVal));
      end;
    finally
      FreeAndNil(tRSL);
    end;
  end;

  procedure SetDayArrayList(const AValue: String; var ODayLsit, OPosList: TList);
  var
    nRCnt, nRLen, nRPos: Integer;
    sRVal: String;
    tRSL: TStringList;
  begin
    ODayLsit.Clear;
    OPosList.Clear;
    if AValue = '' then Exit;
    tRSL := TStringList.Create;
    try
      tRSL.CommaText := AValue;
      for nRCnt := 0 to tRSL.Count - 1 do
      begin
        nRPos := 0;
        sRVal := tRSL.Strings[nRCnt];
        nRLen := Length(sRVal);
        if nRLen > 1 then
        begin
          if sRVal[1] in ['1'..'9'] then
          begin
            nRPos := StrToInt(sRVal[1]);
            sRVal := Copy(sRVal, 2, nRLen);
          end;
          sRVal := AnsiUpperCase(sRVal);
          if sRVal = '' then
            Continue  //Ȃ
          else if (AnsiPos('MO', sRVal) = 1)or(AnsiPos('', sRVal) = 1) then
            ODayLsit.Add(PInteger(DayMonday))
          else if (AnsiPos('TU', sRVal) = 1)or(AnsiPos('', sRVal) = 1) then
            ODayLsit.Add(PInteger(DayTuesday))
          else if (AnsiPos('WE', sRVal) = 1)or(AnsiPos('', sRVal) = 1) then
            ODayLsit.Add(PInteger(DayWednesday))
          else if (AnsiPos('TH', sRVal) = 1)or(AnsiPos('', sRVal) = 1) then
            ODayLsit.Add(PInteger(DayThursday))
          else if (AnsiPos('FR', sRVal) = 1)or(AnsiPos('', sRVal) = 1) then
            ODayLsit.Add(PInteger(DayFriday))
          else if (AnsiPos('SA', sRVal) = 1)or(AnsiPos('y', sRVal) = 1) then
            ODayLsit.Add(PInteger(DaySaturday))
          else if (AnsiPos('SU', sRVal) = 1)or(AnsiPos('', sRVal) = 1) then
            ODayLsit.Add(PInteger(DaySunday));
          OPosList.Add(PInteger(nRPos));
        end;
      end;
    finally
      FreeAndNil(tRSL);
    end;
  end;
begin
  if FUpdateLocked then Exit;
  for nCnt := 0 to FMatchingList.Count - 1 do
  begin
    TScheduleItem(FMatchingList.Items[nCnt]).Free;
  end;
  FMatchingList.Clear;

  tDS := TCsvDataSet.Create;
  try
    OpenData(tDS);
    tDS.First;
    while not tDS.Eof do
    begin
      tCI := TScheduleItem.Create;

      nIdx := tDS.GetFieldIndex('SUMMARY');
      if nIdx >= 0 then tCI.SUMMARY := tDS.FieldByIndex(nIdx).AsString;

      nIdx := tDS.GetFieldIndex('DESCRIPTION');
      if nIdx >= 0 then tCI.DESCRIPTION := tDS.FieldByIndex(nIdx).AsString;

      nIdx := tDS.GetFieldIndex('CATEGORIES');
      if nIdx >= 0 then
      begin
        sVal := AnsiUpperCase(tDS.FieldByIndex(nIdx).AsString);
        if (sVal = '') then
          tCI.CATEGORIES := todWeekday
        else if (AnsiPos('SPECIAL', sVal) > 0) then
          tCI.CATEGORIES := todSpecial
        else if (AnsiPos('HALFDAY', sVal) > 0)or(AnsiPos('x', sVal) > 0) then
          tCI.CATEGORIES := todHalfday
        else if (AnsiPos('HOLIDAY', sVal) > 0)or(AnsiPos('j', sVal) > 0)or(AnsiPos('x', sVal) > 0) then
          tCI.CATEGORIES := todHoliday
        else if (AnsiPos('WEEKDAY', sVal) > 0)or(AnsiPos('', sVal) > 0) then
          tCI.CATEGORIES := todWeekday
        else
          tCI.CATEGORIES := todSpecial;
      end;

      nIdx := tDS.GetFieldIndex('DTSTART');
      if nIdx >= 0 then tCI.DTSTART := DTStampToJulianDef(tDS.FieldByIndex(nIdx).AsString, MinJulianDay);

      nIdx := tDS.GetFieldIndex('DTEND');
      if nIdx >= 0 then tCI.DTEND := DTStampToJulianDef(tDS.FieldByIndex(nIdx).AsString, MinJulianDay);
      if tCI.DTSTART >= tCI.DTEND then tCI.DTEND := tCI.DTSTART + 1;

      nIdx := tDS.GetFieldIndex('RRULE');
      if nIdx >= 0 then
      begin
        tSL := TStringList.Create;
        try
          tSL.Delimiter := ';';
          tSL.DelimitedText := AnsiUpperCase(tDS.FieldByIndex(nIdx).AsString);

          nIdx := tSL.IndexOfName('FREQ');
          if nIdx >= 0 then sVal := tSL.Values['FREQ'];
          if (sVal = '') or SameText(sVal, 'NONE') then
            tCI.RULE_FREQ := rfONCE
          else if SameText(sVal, 'YEARLY') or SameText(sVal, 'N') then
            tCI.RULE_FREQ := rfYEARLY
          else if SameText(sVal, 'MONTHLY')  or SameText(sVal, '') then
            tCI.RULE_FREQ := rfMONTHLY
          else if SameText(sVal, 'WEEKLY')  or SameText(sVal, 'T') then
            tCI.RULE_FREQ := rfWEEKLY
          else if SameText(sVal, 'DAILY')  or SameText(sVal, '') then
            tCI.RULE_FREQ := rfDAILY
          else
            tCI.RULE_FREQ := rfONCE;

          nIdx := tSL.IndexOfName('INTERVAL');
          if nIdx >= 0 then tCI.RULE_INTERVAL := StrToIntDef(tSL.Values['INTERVAL'], 0);

          nIdx := tSL.IndexOfName('BYDAY');
          if nIdx >= 0 then SetDayArrayList(tSL.Values['BYDAY'], tCI.RULE_BYDAY, tCI.RULE_BYDAYPOS);

          nIdx := tSL.IndexOfName('BYMONTHDAY');
          if nIdx >= 0 then SetIntArrayList(tSL.Values['BYMONTHDAY'], tCI.RULE_BYMONTHDAY);

          nIdx := tSL.IndexOfName('BYYEARDAY');
          if nIdx >= 0 then SetIntArrayList(tSL.Values['BYYEARDAY'], tCI.RULE_BYYEARDAY);

          nIdx := tSL.IndexOfName('BYWEEKNO');
          if nIdx >= 0 then SetIntArrayList(tSL.Values['BYWEEKNO'], tCI.RULE_BYWEEKNO);

          nIdx := tSL.IndexOfName('BYMONTH');
          if nIdx >= 0 then SetIntArrayList(tSL.Values['BYMONTH'], tCI.RULE_BYMONTH);

          nIdx := tSL.IndexOfName('BYSETPOS');
          if nIdx >= 0 then SetIntArrayList(tSL.Values['BYSETPOS'], tCI.RULE_BYSETPOS);

          nIdx := tSL.IndexOfName('UNTIL');
          if nIdx >= 0 then tCI.RULE_UNTIL := DTStampToJulianDef(tSL.Values['UNTIL'], MaxJulianDay);
        finally
          FreeAndNil(tSL);
        end;
      end;

      FMatchingList.Add(tCI);
      tDS.Next;
    end;
    tDS.Close;
  finally
    FreeAndNil(tDS);
  end;
end;

//J_[ĕ`
procedure TSchedules.Repaint;
var
  Cnt: Integer;
begin
  for Cnt := 0 to FClientList.Count - 1 do
  begin
    if Assigned(FClientList.Items[Cnt]) then
    begin
      TCalendarBase(FClientList.Items[Cnt]).Repaint;
    end;
  end;
end;

//J_[
procedure TSchedules.Synchronize(const Value: LongInt);
var
  Cnt: Integer;
begin
  if FUpdateLocked then Exit;
  FUpdateLocked := True;
  try
    for Cnt := 0 to FClientList.Count - 1 do
    begin
      if Assigned(FClientList.Items[Cnt]) then
      begin
        if TCalendarBase(FClientList.Items[Cnt]).SynchroDate then
        begin
          TCalendarBase(FClientList.Items[Cnt]).JulianDate := Value;
        end;
      end;
    end;
  finally
    FUpdateLocked := False;
  end;
end;

//J_[o^
procedure TSchedules.RegistClient(AClient: TCalendarBase);
var
  Cnt: Integer;
begin
  for Cnt := 0 to FClientList.Count - 1 do
  begin
    if FClientList.Items[Cnt] = AClient then Exit;
  end;
  FClientList.Add(AClient);
end;

//J_[
procedure TSchedules.UnRegistClient(AClient: TCalendarBase);
var
  Cnt: Integer;
begin
  for Cnt := 0 to FClientList.Count - 1 do
  begin
    if FClientList.Items[Cnt] = AClient then
    begin
      FClientList.Delete(Cnt);
      Exit;
    end;
  end;
end;

//Tʐݒ
procedure TSchedules.SetTypeOfWeek(Index: Integer; const Value: TTypeOfDay);
begin
  if FTypeOfDays[Index] = Value then Exit;
  FTypeOfDays[Index] := Value;
  Repaint;
end;

//Tʎ擾
function TSchedules.GetTypeOfWeek(Index: Integer): TTypeOfDay;
begin
  Result := FTypeOfDays[Index];
end;

//tʎ擾
function TSchedules.GetTypeOfDate(JD: LongInt): TTypeOfDays;
begin
  //tXgƃ}b`O
  Result := MatchItem(JD);

  //jɂx`FbN
  Include(Result, GetTypeOfWeek(DayOfTheWeekJ(JD)));
end;

//teLXg擾
function TSchedules.GetTextOfDate(JD: LongInt; WithDetail: Boolean): String;
var
  nCnt: Integer;
  sTxt: String;
  tCL: TScheduleItem;
  nYJ, nMJ, nDJ: Word;
begin
  Result := '';
  DecodeJulianDay(JD, nYJ, nMJ, nDJ);

  for nCnt := 0 to FMatchingList.Count - 1 do
  begin
    tCL := TScheduleItem(FMatchingList.Items[nCnt]);
    sTxt := Trim(tCL.ItemText[WithDetail]);
    if sTxt <> '' then
    begin
      if tCL.Matching(JD, nYJ, nMJ, nDJ) then
      begin
        if Result <> '' then Result := Result + '\n';
        Result := Result + sTxt;
      end;
    end;
  end;

  if Result <> '' then
  begin
    if WithDetail then
      Result := StringReplace(Result, '\n', #13#10, [rfReplaceAll, rfIgnoreCase])
    else
      Result := StringReplace(Result, '\n', '', [rfReplaceAll, rfIgnoreCase]);
  end;
end;

//}b`O
function TSchedules.MatchItem(JD: LongInt): TTypeOfDays;
var
  Cnt: Integer;
  Item: TScheduleItem;
  nYJ, nMJ, nDJ: Word;
begin
  Result := [];
  DecodeJulianDay(JD, nYJ, nMJ, nDJ);

  for Cnt := 0 to FMatchingList.Count - 1 do
  begin
    Item := TScheduleItem(FMatchingList.Items[Cnt]);
    if Item.Matching(JD, nYJ, nMJ, nDJ) then Include(Result, Item.TypeOfDay);
  end;
end;

//t͈̓`FbN
function TSchedules.CheckDateMinMax(const Value: LongInt): LongInt;
begin
  Result := Value;
  if Value < FMinDate then Result := FMinDate;
  if Value > FMaxDate then Result := FMaxDate;
end;


////////////////////////////////////////////////////////////////////////////////
// TCalendarFonts                                                             //
////////////////////////////////////////////////////////////////////////////////

//Create
constructor TCalendarFonts.Create(AOwner: TCalendarBase);
begin
  inherited Create;
  FOwner := AOwner;
  FTitleText := TFont.Create;
  FTitleText.Assign(FOwner.Font);

  FWeekday := DEF_COLOR_WEEKDAY;
  FHalfday := DEF_COLOR_HALFDAY;
  FHoliday := DEF_COLOR_HOLIDAY;
  FSpecial := DEF_COLOR_SPECIAL;
  FTitleBack := DEF_COLOR_TITLE_BACK;
  FTitleText.Color := DEF_COLOR_TITLE_TEXT;
  FTitleText.Style := [fsBold];

  FTitleText.OnChange := TitleFontChanged;
end;

//Destroy
destructor TCalendarFonts.Destroy;
begin
  FreeAndNil(FTitleText);
  inherited;
end;

//^CgtHgύXCxg
procedure TCalendarFonts.TitleFontChanged(Sender: TObject);
begin
  FOwner.TitleResize;
  FOwner.Repaint;
end;

//ttHgJ_[̃tHg擾
function TCalendarFonts.GetDaysText: TFont;
begin
  Result := FOwner.Font;
end;

//ttHgJ_[̃tHgݒ
procedure TCalendarFonts.SetDaysText(const Value: TFont);
begin
  FOwner.Font := Value;
end;

//twiFJ_[̔wiF擾
function TCalendarFonts.GetDaysBack: TColor;
begin
  Result := FOwner.Color;
end;

//twiFJ_[̔wiFݒ
procedure TCalendarFonts.SetDaysBack(const Value: TColor);
begin
  FOwner.Color := Value;
end;

//^CgwiFݒ
procedure TCalendarFonts.SetTitleBack(const Value: TColor);
begin
  if FTitleBack = Value then Exit;
  FTitleBack := Value;
  FOwner.Repaint;
end;

//tʖ̕F擾
function TCalendarFonts.GetColors(Index: TTypeOfDay): TColor;
begin
  case Index of
  todWeekday: Result := FWeekday;
  todHalfday: Result := FHalfday;
  todHoliday: Result := FHoliday;
  todSpecial: Result := FSpecial;
  else Result := FOwner.Font.Color;
  end;
end;

//tʖ̕Fݒ
procedure TCalendarFonts.SetColors(Index: TTypeOfDay; const Value: TColor);
begin
  if GetColors(Index) = Value then Exit;
  case Index of
  todWeekday: FWeekday := Value;
  todHalfday: FHalfday := Value;
  todHoliday: FHoliday := Value;
  todSpecial: FSpecial := Value;
  else FOwner.Font.Color := Value;
  end;
  FOwner.Repaint;
end;


////////////////////////////////////////////////////////////////////////////////
// TCalendarBase                                                              //
////////////////////////////////////////////////////////////////////////////////

//Create
constructor TCalendarBase.Create(AOwner: TComponent);
begin
  inherited;
  TabStop := True;
  ParentFont := False;
  FJulianDate := DateToJulianDay(Today);
  FSyncDate := False;
  FShowTitle := True;
  FCalFonts := TCalendarFonts.Create(Self);
  FFirstDayOfWeek := dowLocaleDefault;
  FPrevBtn := nil;
  FNextBtn := nil;
  Width := 185;
  Height := 185;
  FloatingDockSiteClass := TCalendarDockForm;
end;

//Destroy
destructor TCalendarBase.Destroy;
begin
  if Assigned(FSchedules) then
  try
    FSchedules.UnRegistClient(Self);
  finally
    FSchedules := nil;
  end;
  FreeAndNil(FCalFonts);
  if Assigned(FNextBtn) then FreeAndNil(FNextBtn);
  if Assigned(FPrevBtn) then FreeAndNil(FPrevBtn);
  inherited;
end;

//CreateParams
procedure TCalendarBase.CreateParams(var Params: TCreateParams);
begin
  inherited;
  Params.Style := Params.Style or WS_TABSTOP;
end;

//WMLButtonDown
procedure TCalendarBase.WMLButtonDown(var Message: TWMLButtonDown);
begin
  if CanFocus then SetFocus;
  inherited;
end;

//WMContextMenu
procedure TCalendarBase.WMContextMenu(var Message: TWMContextMenu);
begin
  if CanFocus then SetFocus;
  inherited;
end;

//WMSize
procedure TCalendarBase.WMSize(var Message: TWMSize);
begin
  inherited;
  TitleResize;
end;

//ړ{^Cxg
procedure TCalendarBase.NextBtnClick(Sender: TObject);
begin
  DoNextBtnClick;
end;

//Oړ{^Cxg
procedure TCalendarBase.PrevBtnClick(Sender: TObject);
begin
  DoPrevBtnClick;
end;

//{^tH[JXŃJ_[ɃtH[JX
procedure TCalendarBase.ParentEnter(Sender: TObject);
begin
  if CanFocus then SetFocus;
end;

//^Cgo[TCYύX
procedure TCalendarBase.TitleResize;
var
  BtnW, BtnH, BtnT: Integer;
begin
  if not GetShowButton then Exit; //\ȂΉȂ
  BtnW := GetTitleHeight;
  BtnH := BtnW - 4;
  BtnT := BtnH div 2;
  FPrevBtn.Font.Height := BtnT;
  FNextBtn.Font.Height := BtnT;
  BtnT := ClientRect.Top + 2;
  FPrevBtn.SetBounds(ClientRect.Left + 2, BtnT, BtnW, BtnH);
  FNextBtn.SetBounds(ClientRect.Right - BtnW - 2, BtnT, BtnW, BtnH);
end;

//}EX{^
procedure TCalendarBase.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  NewDate: LongInt;
begin
  inherited;
  if GetMousePointDate(X, Y, NewDate) then
  begin
    JulianDate := NewDate;
    DoDateSelect;
  end;
end;

//}EX|C^t擾
function TCalendarBase.GetMousePointDate(X, Y: Integer; out NewValue: LongInt): Boolean;
begin
  Result := False;
end;

//DateTimePickExp̓CxgR[iExitj
procedure TCalendarBase.DoExit;
begin
  if Assigned(FOnInternalExit) then FOnInternalExit(Self);
  inherited;
end;

//DateTimePickExp̓CxgR[iDateChangej
procedure TCalendarBase.DoDateChange;
begin
  if Assigned(FOnInternalChange) then FOnInternalChange(Self);
  if Assigned(FOnDateChange) then FOnDateChange(Self);
  if SynchroDate then
  begin
    if Assigned(FSchedules) then FSchedules.Synchronize(JulianDate);
  end;
end;

//DateTimePickExp̓CxgR[iDateSelectj
procedure TCalendarBase.DoDateSelect;
begin
  if Assigned(FOnInternalSelect) then FOnInternalSelect(Self);
  if Assigned(FOnDateSelect) then FOnDateSelect(Self);
end;

//{^CxgR[
procedure TCalendarBase.DoNextBtnClick;
begin
  if Assigned(FOnNextBtnClick) then FOnNextBtnClick(Self);
end;

//O{^CxgR[
procedure TCalendarBase.DoPrevBtnClick;
begin
  if Assigned(FOnPrevBtnClick) then FOnPrevBtnClick(Self);
end;

//TJnj擾
function TCalendarBase.ConvFirstDayOfWeek: Byte;
var
  DOWFlag: Byte;
  C: array[0..1] of Char;
begin
  //TCalDayOfWeek1`7̃tOɕϊi1:j`7:jj
  if FirstDayOfWeek <> dowLocaleDefault then
    DOWFlag := Ord(FirstDayOfWeek)
  else if GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_IFIRSTDAYOFWEEK, C, SizeOf(C)) <> 0 then
    DOWFlag := Ord(C[0]) - Ord('0')
  else
    DOWFlag := 0;
  Result := DOWFlag + 1;
end;

//tʖ̃tHg擾
function TCalendarBase.GetFontOfTOD(TOD: TTypeOfDays): TColor;
begin
  if todSpecial in TOD then
    Result := FCalFonts.Special
  else if todHoliday in TOD then
    Result := FCalFonts.Holiday
  else if todHalfday in TOD then
    Result := FCalFonts.Halfday
  else if todWeekday in TOD then
    Result := FCalFonts.Weekday
  else
    Result := FCalFonts.DaysText.Color;
end;

//j̃tHg擾
function TCalendarBase.GetFontOfWeek(DOW: Byte): TColor;
begin
  if Assigned(FSchedules) then
    Result := GetFontOfTOD([FSchedules.GetTypeOfWeek(DOW)])
  else
    Result := FCalFonts.DaysText.Color;
end;

//t̃tHg擾
function TCalendarBase.GetFontOfDate(JD: LongInt): TColor;
begin
  if Assigned(FSchedules) then
    Result := GetFontOfTOD(FSchedules.GetTypeOfDate(JD))
  else
    Result := FCalFonts.DaysText.Color;
end;

//teLXg擾
function TCalendarBase.GetTextOfDate(JD: LongInt; MultiLine: Boolean): String;
begin
  Result := '';
  if not Assigned(FSchedules) then Exit;
  Result := FSchedules.GetTextOfDate(JD, MultiLine);
end;

//J_[tHgݒ
procedure TCalendarBase.SetCalendarFonts(const Value: TCalendarFonts);
begin
  FCalFonts.Assign(Value);
  Repaint;
end;

//J_[t擾
function TCalendarBase.GetCurrentDate: TDateTime;
begin
  Result := JulianDayToDate(JulianDate);
end;

//J_[N擾
function TCalendarBase.GetCurrentYear: Word;
var
  Y, M, D: Word;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  Result := Y;
end;

//J_[擾
function TCalendarBase.GetCurrentMonth: Word;
var
  Y, M, D: Word;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  Result := M;
end;

//J_[擾
function TCalendarBase.GetCurrentDay: Word;
var
  Y, M, D: Word;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  Result := D;
end;

//{^\敪擾
function TCalendarBase.GetShowButton: Boolean;
begin
  Result := Assigned(FPrevBtn) and Assigned(FNextBtn);
end;

//J_[tݒ
procedure TCalendarBase.SetCurrentDate(const Value: TDateTime);
begin
  JulianDate := DateToJulianDay(Value);
end;

//J_[Nݒ
procedure TCalendarBase.SetCurrentYear(const Value: Word);
var
  Y, M, D: Word;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  JulianDate := EncodeJulianDay(Value, M, D);
end;

//J_[ݒ
procedure TCalendarBase.SetCurrentMonth(const Value: Word);
var
  Y, M, D: Word;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  JulianDate := EncodeJulianDay(Y, Value, D);
end;

//J_[ݒ
procedure TCalendarBase.SetCurrentDay(const Value: Word);
var
  Y, M, D: Word;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  JulianDate := EncodeJulianDay(Y, M, Value);
end;

//J_[EXݒ
procedure TCalendarBase.SetJulianDate(const Value: LongInt);
var
  TempDate: LongInt;
begin
  if Assigned(FSchedules) then
    TempDate := FSchedules.CheckDateMinMax(Value)
  else
    TempDate := Value;
  if FJulianDate = TempDate then Exit;
  FJulianDate := TempDate;
  Repaint;
  DoDateChange;
end;

//TJnjݒ
procedure TCalendarBase.SetFirstDayOfWeek(const Value: TCalDayOfWeek);
begin
  if FFirstDayOfWeek = Value then Exit;
  FFirstDayOfWeek := Value;
  Repaint;
end;

//{^\敪ݒ
procedure TCalendarBase.SetShowButton(const Value: Boolean);
  function CreateButton(ANext: Boolean): TRapidBtnLite;
  begin
    //Oړ{^
    Result := TRapidBtnLite.Create(Self);
    if ANext then
    begin
      Result.Anchors := [akTop, akRight];
      Result.Caption := '>';
      Result.OnClick := NextBtnClick;
    end
    else
    begin
      Result.Anchors := [akLeft, akTop];
      Result.Caption := '<';
      Result.OnClick := PrevBtnClick;
    end;
    Result.OnEnter := ParentEnter;
    Result.Visible := True;
    Result.Enabled := True;
    Result.Parent := Self;
  end;
begin
  if Assigned(FNextBtn) <> Value then
  begin
    if Value then
      FNextBtn := CreateButton(True)
    else
      FreeAndNil(FNextBtn);
  end;
  if Assigned(FPrevBtn) <> Value then
  begin
    if Value then
      FPrevBtn := CreateButton(False)
    else
      FreeAndNil(FPrevBtn);
  end;
  if Value then TitleResize;
  Repaint;
end;

//^Cg\敪ݒ
procedure TCalendarBase.SetShowTitle(const Value: Boolean);
begin
  if FShowTitle = Value then Exit;
  FShowTitle := Value;
  Repaint;
end;

//t敪ݒ
procedure TCalendarBase.SetSynchroDate(const Value: Boolean);
begin
  if FSyncDate = Value then Exit;
  FSyncDate := Value;
  if Value then
  begin
    if Assigned(FSchedules) then FSchedules.Synchronize(JulianDate);
  end;
end;

//XPW[ݒ
procedure TCalendarBase.SetSchedules(Value: TSchedules);
begin
  if FSchedules = Value then Exit;
  if Assigned(FSchedules) then
  begin
    FSchedules.UnRegistClient(Self);
    FSchedules := nil;
  end;
  if Assigned(Value) then
  begin
    FSchedules := Value;
    FSchedules.RegistClient(Self);
  end;
  Repaint;
end;

//tړ
procedure TCalendarBase.JumpTo(Y, M, D: Word);
var
  MaxD: Word;
begin
  MaxD := DaysInAMonth(Y, M);
  if MaxD < D then D := MaxD;
  JulianDate := EncodeJulianDay(Y, M, D);
end;

//ݓړ
procedure TCalendarBase.JumpToToday;
begin
  JulianDate := DateToJulianDay(Today);
end;

//ړ
procedure TCalendarBase.JumpToNextDay;
begin
  JulianDate := JulianDate + 1;
end;

//Oړ
procedure TCalendarBase.JumpToPrevDay;
begin
  JulianDate := JulianDate - 1;
end;

//Tړ
procedure TCalendarBase.JumpToNextWeek;
begin
  JulianDate := JulianDate + 7;
end;

//Tړ
procedure TCalendarBase.JumpToPrevWeek;
begin
  JulianDate := JulianDate - 7;
end;

//ړ
procedure TCalendarBase.JumpToNextMonth;
var
  Y, M, D: Word;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  if M = 12 then
  begin
    M := 1;
    Inc(Y);
  end
  else
    Inc(M);
  JumpTo(Y, M, D);
end;

//Oړ
procedure TCalendarBase.JumpToPrevMonth;
var
  Y, M, D: Word;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  if M = 1 then
  begin
    M := 12;
    Dec(Y);
  end
  else
    Dec(M);
  JumpTo(Y, M, D);
end;

//Nړ
procedure TCalendarBase.JumpToNextYear;
var
  Y, M, D: Word;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  JumpTo(Y + 1, M, D);
end;

//ONړ
procedure TCalendarBase.JumpToPrevYear;
var
  Y, M, D: Word;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  JumpTo(Y - 1, M, D);
end;

//wړ
procedure TCalendarBase.JumpToIncMonth(Months: Word);
var
  Y, M, D: Word;
  YY, MM: Integer;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  YY := Y + (Months div 12);
  MM := M + (Months mod 12);
  if MM > 12 then
  begin
    Inc(YY);
    Dec(MM, 12);
  end;
  JumpTo(YY, MM, D);
end;

//wړ
procedure TCalendarBase.JumpToDecMonth(Months: Word);
var
  Y, M, D: Word;
  YY, MM: Integer;
begin
  DecodeJulianDay(JulianDate, Y, M, D);
  YY := Y - Months div 12;
  MM := M - Months mod 12;
  if MM < 1 then
  begin
    Dec(YY);
    Inc(MM, 12);
  end;
  JumpTo(YY, MM, D);
end;

//Tړ
procedure TCalendarBase.JumpToStartOfWeek;
begin
  JulianDate := StartOfTheWeekJ(JulianDate, ConvFirstDayOfWeek);
end;

//Tړ
procedure TCalendarBase.JumpToEndOfWeek;
begin
  JulianDate := EndOfTheWeekJ(JulianDate, ConvFirstDayOfWeek);
end;

//ړ
procedure TCalendarBase.JumpToStartOfMonth;
begin
  JulianDate := StartOfTheMonthJ(JulianDate);
end;

//ړ
procedure TCalendarBase.JumpToEndOfMonth;
begin
  JulianDate := EndOfTheMonthJ(JulianDate);
end;

//Nړ
procedure TCalendarBase.JumpToStartOfHalf;
begin
  JulianDate := StartOfTheHalfJ(JulianDate);
end;

//Nړ
procedure TCalendarBase.JumpToEndOfHalf;
begin
  JulianDate := EndOfTheHalfJ(JulianDate);
end;

//Nړ
procedure TCalendarBase.JumpToStartOfYear;
begin
  JulianDate := StartOfTheYearJ(JulianDate);
end;

//Nړ
procedure TCalendarBase.JumpToEndOfYear;
begin
  JulianDate := EndOfTheYearJ(JulianDate);
end;

//^Cgo[擾
function TCalendarBase.GetTitleHeight: Integer;
begin
  if Assigned(FCalFonts) then
    Result := Abs(FCalFonts.TitleText.Height) * 2 + 4
  else
    Result := 32;
end;

//^Cgo[`
procedure TCalendarBase.DrawTitle(ATitle: String; ACanvas: TCanvas; ARect: TRect);
begin
  if FCalFonts.TitleBack <> clDefault then
  begin
    ACanvas.Brush.Style := bsSolid;
    ACanvas.Brush.Color := FCalFonts.TitleBack;
    ACanvas.FillRect(ARect);
  end;

  if ATitle <> '' then
  begin
    ACanvas.Brush.Style := bsClear;
    ACanvas.Font.Assign(FCalFonts.TitleText);
    DrawText(ACanvas.Handle, PChar(ATitle), -1, ARect,
             DT_NOCLIP or DT_SINGLELINE or DT_VCENTER or DT_CENTER);
  end;
end;


////////////////////////////////////////////////////////////////////////////////
// TCalendarDockForm                                                          //
////////////////////////////////////////////////////////////////////////////////

constructor TCalendarDockForm.Create(AOwner: TComponent);
begin
  CreateNew(AOwner);
  AutoScroll := False;
  BorderStyle := bsNone;
  FormStyle := fsStayOnTop;
end;

procedure TCalendarDockForm.DoAddDockClient(Client: TControl; const ARect: TRect);
begin
  inherited DoAddDockClient(Client, ARect);
  Client.Align := alClient;
end;

procedure TCalendarDockForm.DoRemoveDockClient(Client: TControl);
begin
  inherited DoRemoveDockClient(Client);
  if DockClientCount = 0 then Release;
end;

procedure TCalendarDockForm.CMDockNotification(var Message: TCMDockNotification);
begin
  inherited;
  case Message.NotifyRec^.ClientMsg of
    CM_VISIBLECHANGED: Visible := Message.Client.Visible;
  end;
end;

procedure TCalendarDockForm.CMUnDockClient(var Message: TCMUnDockClient);
begin
  inherited;
  Message.Client.Align := alNone;
end;

end.
///////////////////////////////////////////////////////// (C)2003-2008 L H
