////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// TFolderDialog \[Xt@C Ver.2.0                                       //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
unit FolderDialog;

{$WARN UNIT_PLATFORM OFF}

interface

uses
  Windows, Classes, Controls, ShlObj, ActiveX, Forms;

type
  TDialogPos = (dpDefault,
                dpMainFormCenter, dpMainFormTopLeft,
                dpOwnerFormCenter, dpOwnerFormTopLeft,
                dpDesktopCenter, dpDesktopTopLeft,
                dpScreenCenter, dpScreenTopLeft);

  TCustomFolderDialog = class(TComponent)
  private
    { Private 錾 }
    FTitle: String;
    FDirectory: String;
    FNewStyle: Boolean;
    FPosition: TDialogPos;
    FPointX: Integer;
    FPointY: Integer;
  protected
    { Protected 錾 }
  public
    { Public 錾 }
    constructor Create(AOwner: TComponent); override;
    function Execute: Boolean;
    property Title: String read FTitle write FTitle;
    property Directory: String read FDirectory write FDirectory;
    property NewStyle: Boolean read FNewStyle write FNewStyle default True;
    property Position: TDialogPos read FPosition write FPosition default dpDefault;
    property PointX: Integer read FPointX write FPointX default 0;
    property PointY: Integer read FPointY write FPointY default 0;
  published
    { Published 錾 }
  end;

  TFolderDialogLite = class(TCustomFolderDialog)
  published
    { Published 錾 }
    property Title;
    property Directory;
    property NewStyle;
  end;

  TFolderDialog = class(TCustomFolderDialog)
  published
    { Published 錾 }
    property Title;
    property Directory;
    property NewStyle;
    property Position;
    property PointX;
    property PointY;
  end;

//tH_Iʊ֐
function PromptForDirectory(var ADirectory: String;
  const ATitle: String = ''; ANewStyle: Boolean = True; AOwner: TWinControl = nil): Boolean;

implementation

////////////////////////////////////////////////////////////////////////////////

//R[obN֐
function BrowseCallbackProc(Wnd: HWND; uMsg: UINT; LParam, lpData: LPARAM): Integer; Stdcall;
var
  pPath: array[0..511] of Char;
  sText: String;

  procedure SetDialogPosition;
  var
    rRect: TRect;
    nLeft, nTop, nWidth, nHeight: Integer;
    tFD: TFolderDialog;

    procedure SetScreenCenter;
    begin
      nLeft := (Screen.Width - nWidth) div 2;
      nTop  := (Screen.Height - nHeight) div 2;
    end;

    procedure SetScreenTopLeft;
    begin
      nLeft := 0;
      nTop  := 0;
    end;
  begin
    if GetWindowRect(Wnd, rRect) then
    try
      tFD := TFolderDialog(lpData);
      nWidth := rRect.Right - rRect.Left;
      nHeight := rRect.Bottom - rRect.Top;

      case tFD.Position of
      dpMainFormCenter, dpMainFormTopLeft:
        if Assigned(Application.MainForm) then
        begin
          nLeft := Application.MainForm.Left;
          nTop  := Application.MainForm.Top;
          if tFD.Position = dpMainFormCenter then
          begin
            nLeft := nLeft + (Application.MainForm.Width - nWidth) div 2;
            nTop  := nTop + (Application.MainForm.Height - nHeight) div 2;
          end;
        end
        else
        begin
          if tFD.Position = dpMainFormCenter then
            SetScreenCenter
          else
            SetScreenTopLeft;
        end;
      dpOwnerFormCenter, dpOwnerFormTopLeft:
        if Assigned(tFD.Owner) and (tFD.Owner is TCustomForm) then
        begin
          nLeft := TCustomForm(tFD.Owner).Left;
          nTop  := TCustomForm(tFD.Owner).Top;
          if tFD.Position = dpOwnerFormCenter then
          begin
            nLeft := nLeft + (TCustomForm(tFD.Owner).Width - nWidth) div 2;
            nTop  := nTop + (TCustomForm(tFD.Owner).Height - nHeight) div 2;
          end;
        end
        else
        begin
          if tFD.Position = dpOwnerFormCenter then
            SetScreenCenter
          else
            SetScreenTopLeft;
        end;
      dpDesktopCenter, dpDesktopTopLeft:
        begin
          nLeft := Screen.DesktopLeft;
          nTop  := Screen.DesktopTop;
          if tFD.Position = dpDesktopCenter then
          begin
            nLeft := nLeft + (Screen.DesktopWidth - nWidth) div 2;
            nTop  := nTop + (Screen.DesktopHeight - nHeight) div 2;
          end;
        end;
      dpScreenCenter:
        SetScreenCenter;
      dpScreenTopLeft:
        SetScreenTopLeft;
      else//dpDefault:
        nLeft := rRect.Left;
        nTop  := rRect.Top;
      end;

      nLeft := nLeft + tFD.PointX;
      nTop  := nTop + tFD.PointY;
      if nLeft < 0 then nLeft := 0;
      if nTop < 0 then nTop := 0;
      MoveWindow(Wnd, nLeft, nTop, nWidth, nHeight, False);
    finally
    end;
  end;
begin
  Result := 0;

  if (uMsg = BFFM_INITIALIZED) or (uMsg = BFFM_SELCHANGED) then
  begin
    pPath := '';
    sText := '';

    case uMsg of
    BFFM_INITIALIZED:
      begin
        //_CAO̕\ʒuݒ
        if TObject(lpData) is TFolderDialog then SetDialogPosition;
        //ftHg̃tH_pXI
        sText := TFolderDialog(lpData).Directory;
        //if not DirectoryExists(sText) then Exit;
        SendMessage(Wnd, BFFM_SETSELECTION, 1, LongInt(PChar(sText)));
      end;
    BFFM_SELCHANGED:
      begin
        //[U[ύXtH_pX擾
        if not SHGetPathFromIDList(PItemIDList(lParam), pPath) then Exit;
        sText := String(pPath);
      end;
    end;

    //ύXꂽtH_pX\
    if sText <> '' then
    begin
      SendMessage(Wnd, BFFM_SETSTATUSTEXT, 0, LongInt(PChar(sText)));
    end;
  end;
end;

//RXgN^
constructor TCustomFolderDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTitle := '';
  FDirectory := '';
  FNewStyle := True;
  FPosition := dpDefault;
  PointX := 0;
  PointY := 0;
end;

//_CAOJ
function TCustomFolderDialog.Execute: Boolean;
var
  pMalloc: IMalloc;
  pBuf: PChar;
  pRoot, pForm: PItemIDList;
  tBI: TBrowseInfo;
  hOwner: HWND;
begin
  Result := False;
  SHGetMalloc(pMalloc);
  try
    pBuf := pMalloc.Alloc(MAX_PATH);
    if pBuf <> nil then
    begin
      //I[i[nh擾
      if Assigned(Owner) and (Owner is TWinControl) then
        hOwner := (Owner as TWinControl).Handle
      else
        hOwner := Application.Handle;

      //[g̃pX擾
      if Succeeded(SHGetSpecialFolderLocation(hOwner, CSIDL_DESKTOP, pRoot)) then
      begin
        //TBrowseInfo\̂ݒ
        tBI.hwndOwner := hOwner;
        tBI.pidlRoot := pRoot;
        tBI.pszDisplayName := pBuf;
        if FTitle = '' then
        begin
          tBI.lpszTitle := PChar('Folder Select');
        end
        else
        begin
          tBI.lpszTitle := PChar(FTitle);
        end;
        tBI.ulFlags := $0004; //BIF_STATUSTEXT;
        if FNewStyle then
        begin
          tBI.ulFlags := tBI.ulFlags or $0040; //BIF_NEWDIALOGSTYLE;
        end;
        tBI.lpfn := BrowseCallbackProc;
        tBI.lParam := LongInt(Self);

        //tH_I_CAO\
        pForm := SHBrowseForFolder(tBI);
        if pForm <> nil then
        begin
          if SHGetPathFromIDList(pForm, pBuf) then
          begin
            FDirectory := String(pBuf);
          end;
          pMalloc.Free(pForm);
          Result := True;
        end;
        pMalloc.Free(pRoot);
      end;
      pMalloc.Free(pBuf);
    end;
  finally
    pMalloc._Release;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

function PromptForDirectory(var ADirectory: String;
  const ATitle: String = ''; ANewStyle: Boolean = True; AOwner: TWinControl = nil): Boolean;
var
  Dialog: TCustomFolderDialog;
begin
  Dialog := TCustomFolderDialog.Create(AOwner);
  with Dialog do
  try
    Title := ATitle;
    Directory := ADirectory;
    NewStyle := ANewStyle;
    Result := Execute;
    if Result then
    begin
      ADirectory := Directory;
    end;
  finally
    Free;
  end;
end;

end.
///////////////////////////////////////////////////////// (C)2003-2008 L H

